@@ -30,13 +30,13 @@
 #include "registry.h"
 #include "introspection.h"
 
-static gboolean
+static void
 children_added_listener (DBusConnection * bus,
                          gint             index,
                          const gchar    * name,
                          const gchar    * path);
 
-static gboolean
+static void
 children_removed_listener (DBusConnection * bus,
                            gint             index,
                            const gchar    * name,
@@ -95,22 +95,26 @@ return_v_string (DBusMessageIter * iter, const gchar * str)
 {
   DBusMessageIter variant;
 
-  dbus_message_iter_open_container (iter, DBUS_TYPE_VARIANT, "s",
-                                    &variant);
+  if (!dbus_message_iter_open_container (iter, DBUS_TYPE_VARIANT, "s",
+                                    &variant))
+    return FALSE;
     dbus_message_iter_append_basic (&variant, DBUS_TYPE_STRING, &str);
   dbus_message_iter_close_container (iter, &variant);
+  return TRUE;
 }
 
-static void
+static dbus_bool_t
 append_reference (DBusMessageIter * iter, const char * name, const char * path)
 {
   DBusMessageIter iter_struct;
 
-  dbus_message_iter_open_container (iter, DBUS_TYPE_STRUCT, NULL,
-                                    &iter_struct);
+  if (!dbus_message_iter_open_container (iter, DBUS_TYPE_STRUCT, NULL,
+                                    &iter_struct))
+    return FALSE;
   dbus_message_iter_append_basic (&iter_struct, DBUS_TYPE_STRING, &name);
   dbus_message_iter_append_basic (&iter_struct, DBUS_TYPE_OBJECT_PATH, &path);
   dbus_message_iter_close_container (iter, &iter_struct);
+  return TRUE;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -182,10 +186,11 @@ set_id (SpiRegistry *reg, DBusConnection *bus, const gchar *name, const gchar *p
 static void
 remove_application (SpiRegistry *reg, DBusConnection *bus, guint index)
 {
-  const gchar *name = "";
-  g_ptr_array_remove_index (reg->apps, index);
+  SpiReference *ref = g_ptr_array_index (reg->apps, index);
+
   /*TODO spi_remove_device_listeners (registry->de_controller, old);*/
-  children_removed_listener (bus, index, name, SPI_DBUS_PATH_NULL);
+  children_removed_listener (bus, index, ref->name, ref->path);
+  g_ptr_array_remove_index (reg->apps, index);
 }
 
 static void
@@ -209,8 +214,8 @@ handle_disconnection (DBusConnection *bus, DBusMessage *message, void *user_data
               SpiReference *ref  = g_ptr_array_index (reg->apps, i);
               while (!g_strcmp0 (old, ref->name))
                 {
+                  children_removed_listener (bus, i, old, ref->path);
                   g_ptr_array_remove_index (reg->apps, i);
-                  children_removed_listener (bus, i, old, SPI_DBUS_PATH_NULL);
                 }
             } 
         }
@@ -465,7 +470,7 @@ impl_get_ChildCount (DBusMessageIter * iter, void *user_data)
   SpiRegistry *reg = SPI_REGISTRY (user_data);
   dbus_int32_t rv = reg->apps->len;
 
-  dbus_message_iter_append_basic (iter, DBUS_TYPE_INT32, &rv);
+  return dbus_message_iter_append_basic (iter, DBUS_TYPE_INT32, &rv);
 }
 
 static DBusMessage *
@@ -489,8 +494,13 @@ impl_GetChildAtIndex (DBusConnection * bus,
   reply = dbus_message_new_method_return (message);
   dbus_message_iter_init_append (reply, &iter);
 
-  ref = g_ptr_array_index (reg->apps, i);
-  append_reference (&iter, ref->name, ref->path);
+  if (i < 0 || i >= reg->apps->len)
+    append_reference (&iter, SPI_DBUS_NAME_REGISTRY, SPI_DBUS_PATH_NULL);
+  else
+    {
+      ref = g_ptr_array_index (reg->apps, i);
+      append_reference (&iter, ref->name, ref->path);
+    }
 
   return reply;
 }
@@ -773,7 +783,7 @@ emit_event (DBusConnection *bus,
  * any_data is the child reference.
  */
 
-static gboolean
+static void
 children_added_listener (DBusConnection * bus,
                          gint             index,
                          const gchar    * name,
@@ -783,14 +793,14 @@ children_added_listener (DBusConnection * bus,
               name, path);
 }
 
-static gboolean
+static void
 children_removed_listener (DBusConnection * bus,
                            gint             index,
                            const gchar    * name,
                            const gchar    * path)
 {
   emit_event (bus, SPI_DBUS_INTERFACE_EVENT_OBJECT, "ChildrenChanged", "remove", index, 0,
-              name, SPI_DBUS_PATH_NULL);
+              name, path);
 }
 
 /*---------------------------------------------------------------------------*/
